@@ -1,6 +1,12 @@
-/* ========================================================================== */
-/* === MATLAB/cholmod_matlab ================================================ */
-/* ========================================================================== */
+//------------------------------------------------------------------------------
+// CHOLMOD/MATLAB/cholmod_matlab.c: utilities for CHOLMOD's MATLAB interface
+//------------------------------------------------------------------------------
+
+// CHOLMOD/MATLAB Module.  Copyright (C) 2005-2022, Timothy A. Davis.
+// All Rights Reserved.
+// SPDX-License-Identifier: GPL-2.0+
+
+//------------------------------------------------------------------------------
 
 /* Utility routines for the CHOLMOD MATLAB mexFunctions.
  *
@@ -15,10 +21,6 @@
 
 #include "cholmod_matlab.h"
 
-#ifndef INT64_T
-#define INT64_T long long
-#endif
-
 /* This file pointer is used for the mread and mwrite mexFunctions.  It must
  * be a global variable, because the file pointer is not passed to the
  * sputil_error_handler function when an error occurs. */
@@ -30,23 +32,17 @@ FILE *sputil_file = NULL ;
 
 /* Define function pointers and other parameters for a mexFunction */
 
-void sputil_config (Int spumoni, cholmod_common *cm)
+void sputil_config (int64_t spumoni, cholmod_common *cm)
 {
     /* cholmod_l_solve must return a real or zomplex X for MATLAB */
     cm->prefer_zomplex = TRUE ;
 
-    /* use mxMalloc and related memory management routines */
-    cm->malloc_memory  = mxMalloc ;
-    cm->free_memory    = mxFree ;
-    cm->realloc_memory = mxRealloc ;
-    cm->calloc_memory  = mxCalloc ;
-
     /* printing and error handling */
     if (spumoni == 0)
     {
 	/* do not print anything from within CHOLMOD */
 	cm->print = -1 ;
-	cm->print_function = NULL ;
+        SuiteSparse_config_printf_func_set (NULL) ;
     }
     else
     {
@@ -55,26 +51,12 @@ void sputil_config (Int spumoni, cholmod_common *cm)
 	 * spumoni = 2: also print a short summary of each object.
 	 */
 	cm->print = spumoni + 2 ;
-	cm->print_function = mexPrintf ;
+        /* SuiteSparse_config_printf_func_set ((void *) mexPrintf) ; */
     }
 
     /* error handler */
     cm->error_handler  = sputil_error_handler ;
 
-    /* complex arithmetic */
-    cm->complex_divide = cholmod_l_divcomplex ;
-    cm->hypotenuse     = cholmod_l_hypot ;
-
-#ifndef NPARTITION
-#if defined(METIS_VERSION)
-#if (METIS_VERSION >= METIS_VER(4,0,2))
-    /* METIS 4.0.2 uses function pointers for malloc and free */
-    METIS_malloc = cm->malloc_memory ;
-    METIS_free   = cm->free_memory ;
-#endif
-#endif
-#endif
-
     /* Turn off METIS memory guard.  It is not needed, because mxMalloc will
      * safely terminate the mexFunction and free any workspace without killing
      * all of MATLAB.  This assumes cholmod_make was used to compile CHOLMOD
@@ -87,7 +69,8 @@ void sputil_config (Int spumoni, cholmod_common *cm)
 /* === sputil_error_handler ================================================= */
 /* ========================================================================== */
 
-void sputil_error_handler (int status, char *file, int line, char *message)
+void sputil_error_handler (int status, const char *file, int line,
+    const char *message)
 {
     if (status < CHOLMOD_OK)
     {
@@ -123,20 +106,20 @@ cholmod_sparse *sputil_get_sparse
     const mxArray *Amatlab, /* MATLAB version of the matrix */
     cholmod_sparse *A,	    /* CHOLMOD version of the matrix */
     double *dummy,	    /* a pointer to a valid scalar double */
-    Int stype		    /* -1: lower, 0: unsymmetric, 1: upper */
+    int64_t stype		    /* -1: lower, 0: unsymmetric, 1: upper */
 )
 {
-    Int *Ap ;
+    int64_t *Ap ;
     A->nrow = mxGetM (Amatlab) ;
     A->ncol = mxGetN (Amatlab) ;
-    A->p = (Int *) mxGetJc (Amatlab) ;
-    A->i = (Int *) mxGetIr (Amatlab) ;
+    A->p = (int64_t *) mxGetJc (Amatlab) ;
+    A->i = (int64_t *) mxGetIr (Amatlab) ;
     Ap = A->p ;
     A->nzmax = Ap [A->ncol] ;
     A->packed = TRUE ;
     A->sorted = TRUE ;
     A->nz = NULL ;
-    A->itype = CHOLMOD_INT ;
+    A->itype = CHOLMOD_LONG ;       /* was CHOLMOD_INT in v1.6 and earlier */
     A->dtype = CHOLMOD_DOUBLE ;
     A->stype = stype ;
 
@@ -262,7 +245,7 @@ cholmod_sparse *sputil_get_sparse_pattern
     cholmod_common *cm
 )
 {
-    cholmod_sparse *A ;
+    cholmod_sparse *A = NULL ;
 
     if (!mxIsSparse (Amatlab))
     {
@@ -296,8 +279,8 @@ cholmod_sparse *sputil_get_sparse_pattern
 	    /* (this is copied and modified from t_cholmod_dense.c) */
 
 	    char *x ;
-	    Int *Ap, *Ai ;
-	    Int nrow, ncol, i, j, nz, nzmax, p ;
+	    int64_t *Ap, *Ai ;
+	    int64_t nrow, ncol, i, j, nz, nzmax, p ;
 
 	    /* -------------------------------------------------------------- */
 	    /* count the number of nonzeros in the result */
@@ -392,14 +375,15 @@ mxArray *sputil_put_sparse
     mxArray *Amatlab ;
     cholmod_sparse *A ;
     A = *Ahandle ;
+    if (!A || A->x == NULL) mexErrMsgTxt ("invalid sparse matrix") ;
     Amatlab = mxCreateSparse (0, 0, 0, 
 	    (A->xtype != CHOLMOD_REAL) ? mxCOMPLEX: mxREAL) ;
     mxSetM (Amatlab, A->nrow) ;
     mxSetN (Amatlab, A->ncol) ;
     mxSetNzmax (Amatlab, A->nzmax) ;
-    mxFree (mxGetJc (Amatlab)) ;
-    mxFree (mxGetIr (Amatlab)) ;
-    mxFree (mxGetPr (Amatlab)) ;
+    MXFREE (mxGetJc (Amatlab)) ;
+    MXFREE (mxGetIr (Amatlab)) ;
+    MXFREE (mxGetPr (Amatlab)) ;
     mxSetJc (Amatlab, A->p) ;
     mxSetIr (Amatlab, A->i) ;
     mxSetPr (Amatlab, A->x) ;
@@ -408,7 +392,8 @@ mxArray *sputil_put_sparse
     mexMakeMemoryPersistent (A->x) ;
     if (A->xtype != CHOLMOD_REAL)
     {
-	mxFree (mxGetPi (Amatlab)) ;
+        if (A->z == NULL) mexErrMsgTxt ("invalid complex sparse matrix") ;
+	MXFREE (mxGetPi (Amatlab)) ;
 	mxSetPi (Amatlab, A->z) ;
 	mexMakeMemoryPersistent (A->z) ;
     }
@@ -438,16 +423,18 @@ mxArray *sputil_put_dense
     mxArray *Amatlab ;
     cholmod_dense *A ;
     A = *Ahandle ;
+    if (!A || A->x == NULL) mexErrMsgTxt ("invalid dense matrix") ;
     Amatlab = mxCreateDoubleMatrix (0, 0,
 	    (A->xtype != CHOLMOD_REAL) ? mxCOMPLEX: mxREAL) ;
     mxSetM (Amatlab, A->nrow) ;
     mxSetN (Amatlab, A->ncol) ;
-    mxFree (mxGetPr (Amatlab)) ;
+    MXFREE (mxGetPr (Amatlab)) ;
     mxSetPr (Amatlab, A->x) ;
     mexMakeMemoryPersistent (A->x) ;
     if (A->xtype != CHOLMOD_REAL)
     {
-	mxFree (mxGetPi (Amatlab)) ;
+        if (A->z == NULL) mexErrMsgTxt ("invalid complex dense matrix") ;
+	MXFREE (mxGetPi (Amatlab)) ;
 	mxSetPi (Amatlab, A->z) ;
 	mexMakeMemoryPersistent (A->z) ;
     }
@@ -462,18 +449,18 @@ mxArray *sputil_put_dense
 /* === sputil_put_int ======================================================= */
 /* ========================================================================== */
 
-/* Convert an Int vector into a double mxArray */
+/* Convert a int64_t vector into a double mxArray */
 
 mxArray *sputil_put_int
 (
-    Int *P,		/* vector to convert */
-    Int n,		/* length of P */
-    Int one_based	/* 1 if convert from 0-based to 1-based, 0 otherwise */
+    int64_t *P,		/* vector to convert */
+    int64_t n,		/* length of P */
+    int64_t one_based	/* 1 if convert from 0-based to 1-based, 0 otherwise */
 )
 {
     double *p ;
     mxArray *Q ;
-    Int i ;
+    int64_t i ;
     Q = mxCreateDoubleMatrix (1, n, mxREAL) ;
     p = mxGetPr (Q) ;
     for (i = 0 ; i < n ; i++)
@@ -492,8 +479,8 @@ mxArray *sputil_put_int
 
 void sputil_error
 (
-    Int error,	    /* kind of error */
-    Int is_index    /* TRUE if a matrix index, FALSE if a matrix dimension */
+    int64_t error,	    /* kind of error */
+    int64_t is_index    /* TRUE if a matrix index, FALSE if a matrix dimension */
 )
 {
     if (error == ERROR_TOO_SMALL)
@@ -545,16 +532,16 @@ void sputil_error
 
 /* convert a double into an integer */
 
-Int sputil_double_to_int   /* returns integer value of x */
+int64_t sputil_double_to_int   /* returns integer value of x */
 (
     double x,	    /* double value to convert */
-    Int is_index,   /* TRUE if a matrix index, FALSE if a matrix dimension */
-    Int n	    /* if a matrix index, x cannot exceed this dimension,
+    int64_t is_index,  /* TRUE if a matrix index, FALSE if a matrix dimension */
+    int64_t n	    /* if a matrix index, x cannot exceed this dimension,
 		     * except that -1 is treated as infinity */
 )
 {
-    Int i ;
-    if (x > INT_MAX)
+    int64_t i ;
+    if (x > INT64_MAX)
     {
 	/* x is way too big for an integer */
 	sputil_error (ERROR_HUGE, is_index) ;
@@ -564,7 +551,7 @@ Int sputil_double_to_int   /* returns integer value of x */
 	/* x must be non-negative */
 	sputil_error (ERROR_TOO_SMALL, is_index) ;
     }
-    i = (Int) x ;
+    i = (int64_t) x ;
     if (x != (double) i)
     {
 	/* x must be an integer */
@@ -592,13 +579,13 @@ Int sputil_double_to_int   /* returns integer value of x */
 /* return the number of elements in an mxArray.  Trigger an error on integer
  * overflow (in case the argument is sparse) */
 
-Int sputil_nelements (const mxArray *arg)
+int64_t sputil_nelements (const mxArray *arg)
 {
     double size ;
-    const Int *dims ;
-    Int k, ndims ;
+    const int64_t *dims ;
+    int64_t k, ndims ;
     ndims = mxGetNumberOfDimensions (arg) ;
-    dims = (Int *) mxGetDimensions (arg) ;
+    dims = (int64_t *) mxGetDimensions (arg) ;
     size = 1 ;
     for (k = 0 ; k < ndims ; k++)
     {
@@ -630,11 +617,11 @@ double sputil_get_double (const mxArray *arg)
 
 /* return an argument as a non-negative integer scalar, or -1 if error */
 
-Int sputil_get_integer
+int64_t sputil_get_integer
 (
     const mxArray *arg,	    /* MATLAB argument to convert */
-    Int is_index,	    /* TRUE if an index, FALSE if a matrix dimension */
-    Int n		    /* maximum value, if an index */
+    int64_t is_index,	    /* TRUE if an index, FALSE if a matrix dimension */
+    int64_t n		    /* maximum value, if an index */
 )
 {
     double x = sputil_get_double (arg) ;
@@ -658,12 +645,12 @@ Int sputil_get_integer
 void sputil_trim
 (
     cholmod_sparse *S,
-    Int k,
+    int64_t k,
     cholmod_common *cm
 )
 {
-    Int *Sp ;
-    Int ncol ;
+    int64_t *Sp ;
+    int64_t ncol ;
     size_t n1, nznew ;
 
     if (S == NULL)
@@ -680,7 +667,7 @@ void sputil_trim
 
     /* reduce S->p in size.  This cannot fail. */
     n1 = ncol + 1 ;
-    S->p = cholmod_l_realloc (k+1, sizeof (Int), S->p, &n1, cm) ;
+    S->p = cholmod_l_realloc (k+1, sizeof (int64_t), S->p, &n1, cm) ;
 
     /* get the new number of entries in S */
     Sp = S->p ;
@@ -707,9 +694,9 @@ cholmod_sparse *sputil_extract_zeros
     cholmod_common *cm
 )
 {
-    Int *Ap, *Ai, *Zp, *Zi ;
+    int64_t *Ap, *Ai, *Zp, *Zi ;
     double *Ax, *Az, *Zx ;
-    Int j, p, nzeros = 0, is_complex, pz, nrow, ncol ;
+    int64_t j, p, nzeros = 0, is_complex, pz, nrow, ncol ;
     cholmod_sparse *Z ;
 
     if (A == NULL || A->xtype == CHOLMOD_PATTERN || A->xtype == CHOLMOD_COMPLEX)
@@ -731,8 +718,8 @@ cholmod_sparse *sputil_extract_zeros
     {
 	for (p = Ap [j] ; p < Ap [j+1] ; p++)
 	{
-	    if (CHOLMOD_IS_ZERO (Ax [p]) &&
-		((is_complex) ? CHOLMOD_IS_ZERO (Az [p]) : TRUE))
+	    if ((Ax [p] == 0) &&
+		((is_complex) ? (Az [p] == 0) : TRUE))
 	    {
 		nzeros++ ;
 	    }
@@ -754,8 +741,8 @@ cholmod_sparse *sputil_extract_zeros
 	    Zp [j] = pz ;
 	    for (p = Ap [j] ; p < Ap [j+1] ; p++)
 	    {
-		if (CHOLMOD_IS_ZERO (Ax [p]) &&
-		    ((is_complex) ? CHOLMOD_IS_ZERO (Az [p]) : TRUE))
+		if ((Ax [p] == 0) &&
+		    ((is_complex) ? (Az [p] == 0) : TRUE))
 		{
 		    Zi [pz] = Ai [p] ;
 		    Zx [pz] = 1 ;
@@ -774,26 +761,27 @@ cholmod_sparse *sputil_extract_zeros
 /* === sputil_drop_zeros ==================================================== */
 /* ========================================================================== */
 
-/* Drop zeros from a CHOLMOD sparse matrix (zomplex or real).  This is very
- * similar to CHOLMOD/MatrixOps/cholmod_drop, except that this routine has
+/* Drop zeros from a packed CHOLMOD sparse matrix (zomplex or real).  This is
+ * very similar to CHOLMOD/MatrixOps/cholmod_drop, except that this routine has
  * no tolerance parameter and it can handle zomplex matrices.  NaN's are left
  * in the matrix.  If this is used on the sparse matrix version of the factor
  * L, then the update/downdate methods cannot be applied to L (ldlupdate).
+ * Returns the number of entries dropped.
  */
 
-void sputil_drop_zeros
+int64_t sputil_drop_zeros
 (
     cholmod_sparse *S
 )
 {
     double sik, zik ;
-    Int *Sp, *Si ;
+    int64_t *Sp, *Si ;
     double *Sx, *Sz ;
-    Int pdest, k, ncol, p, pend ;
+    int64_t pdest, k, ncol, p, pend, nz ;
 
     if (S == NULL)
     {
-	return ;
+	return (0) ;
     }
 
     Sp = S->p ;
@@ -802,6 +790,7 @@ void sputil_drop_zeros
     Sz = S->z ;
     pdest = 0 ;
     ncol = S->ncol ;
+    nz = Sp [ncol] ;
 
     if (S->xtype == CHOLMOD_ZOMPLEX)
     {
@@ -814,7 +803,7 @@ void sputil_drop_zeros
 	    {
 		sik = Sx [p] ;
 		zik = Sz [p] ;
-		if (CHOLMOD_IS_NONZERO (sik) || CHOLMOD_IS_NONZERO (zik))
+		if ((sik != 0) || (zik != 0))
 		{
 		    if (p != pdest)
 		    {
@@ -837,7 +826,7 @@ void sputil_drop_zeros
 	    for ( ; p < pend ; p++)
 	    {
 		sik = Sx [p] ;
-		if (CHOLMOD_IS_NONZERO (sik))
+		if ((sik != 0))
 		{
 		    if (p != pdest)
 		    {
@@ -850,6 +839,7 @@ void sputil_drop_zeros
 	}
     }
     Sp [ncol] = pdest ;
+    return (nz - pdest) ;
 }
 
 
@@ -857,23 +847,29 @@ void sputil_drop_zeros
 /* === sputil_copy_ij ======================================================= */
 /* ========================================================================== */
 
-/* copy i or j arguments into an Int vector.  For small integer types, i and
+/* copy i or j arguments into an int64_t vector.  For small integer types, i and
  * and j can be returned with negative entries; this error condition is caught
  * later, in cholmod_triplet_to_sparse.
+ *
+ * Future work: if the mxClassID matches the default integer (INT32 for 32-bit
+ * MATLAB and INT64 for 64-bit), then it would save memory to patch in the
+ * vector with a pointer copy, rather than making a copy of the whole vector.
+ * This would require that the 1-based i and j vectors be converted on the fly
+ * to 0-based vectors in cholmod_triplet_to_sparse.
  */
 
-Int sputil_copy_ij		/* returns the dimension, n */
+int64_t sputil_copy_ij		/* returns the dimension, n */
 (
-    Int is_scalar,	/* TRUE if argument is a scalar, FALSE otherwise */
-    Int scalar,		/* scalar value of the argument */
+    int64_t is_scalar,	/* TRUE if argument is a scalar, FALSE otherwise */
+    int64_t scalar,        /* scalar value of the argument */
     void *vector,	/* vector value of the argument */
     mxClassID category,	/* type of vector */
-    Int nz,		/* length of output vector I */
-    Int n,		/* maximum dimension, EMPTY if not yet known */
-    Int *I		/* vector of length nz to copy into */
+    int64_t nz,		/* length of output vector I */
+    int64_t n,		/* maximum dimension, EMPTY if not yet known */
+    int64_t *I		/* vector of length nz to copy into */
 )
 {
-    Int i, k, ok, ok2, ok3, n2 ;
+    int64_t i, k, ok, ok2, ok3, n2 ;
 
     if (is_scalar)
     {
@@ -890,7 +886,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
     }
     else
     {
-	/* copy double input into Int vector (convert to 0-based) */
+	/* copy double input into int64_t vector (convert to 0-based) */
 	ok = TRUE ;
 	ok2 = TRUE ;
 	ok3 = TRUE ;
@@ -906,7 +902,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    i = (Int) (((mxLogical *) vector) [k]) ;
+		    i = (int64_t) (((mxLogical *) vector) [k]) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -918,7 +914,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    i = (Int) (((mxChar *) vector) [k]) ;
+		    i = (int64_t) (((mxChar *) vector) [k]) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -928,7 +924,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    i = (Int) (((INT8_T *) vector) [k]) ;
+		    i = (int64_t) (((INT8_T *) vector) [k]) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -938,7 +934,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    i = (Int) (((UINT8_T *) vector) [k]) ;
+		    i = (int64_t) (((UINT8_T *) vector) [k]) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -948,7 +944,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    i = (Int) (((INT16_T *) vector) [k]) ;
+		    i = (int64_t) (((INT16_T *) vector) [k]) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -958,7 +954,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    i = (Int) (((UINT16_T *) vector) [k]) ;
+		    i = (int64_t) (((UINT16_T *) vector) [k]) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -968,7 +964,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    i = (Int) (((INT32_T *) vector) [k]) ;
+		    i = (int64_t) (((INT32_T *) vector) [k]) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -978,8 +974,9 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; ok3 && k < nz ; k++)
 		{
-		    double y = (Int) (((UINT32_T *) vector) [k]) ;
-		    ok3 = (y < Int_max) ;
+		    double y = (((UINT32_T *) vector) [k]) ;
+		    i = (int64_t) y ;
+		    ok3 = (y < INT64_MAX) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -989,10 +986,10 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; ok2 && ok3 && k < nz ; k++)
 		{
-		    INT64_T y = ((INT64_T *) vector) [k] ;
-		    i = (Int) y ;
+		    int64_t y = ((int64_t *) vector) [k] ;
+		    i = (int64_t) y ;
 		    ok2 = (y > 0) ;
-		    ok3 = (y < Int_max) ;
+		    ok3 = (y < INT64_MAX) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -1002,10 +999,10 @@ Int sputil_copy_ij		/* returns the dimension, n */
 
 		for (k = 0 ; ok2 && ok3 && k < nz ; k++)
 		{
-		    unsigned INT64_T y = ((unsigned INT64_T *) vector) [k] ;
-		    i = (Int) y ;
+		    uint64_t y = ((uint64_t *) vector) [k] ;
+		    i = (int64_t) y ;
 		    ok2 = (y > 0) ;
-		    ok3 = (y < Int_max) ;
+		    ok3 = (y < INT64_MAX) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -1016,10 +1013,10 @@ Int sputil_copy_ij		/* returns the dimension, n */
 		for (k = 0 ; ok && ok2 && ok3 && k < nz ; k++)
 		{
 		    float y = ((float *) vector) [k] ;
-		    i = (Int) y ;
+		    i = (int64_t) y ;
 		    ok = (y == (float) i) ;
 		    ok2 = (y > 0) ;
-		    ok3 = (y < Int_max) ;
+		    ok3 = (y < INT64_MAX) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -1030,10 +1027,10 @@ Int sputil_copy_ij		/* returns the dimension, n */
 		for (k = 0 ; ok && ok2 && ok3 && k < nz ; k++)
 		{
 		    double y = ((double *) vector) [k] ;
-		    i = (Int) y ;
+		    i = (int64_t) y ;
 		    ok = (y == (double) i) ;
 		    ok2 = (y > 0) ;
-		    ok3 = (y < Int_max) ;
+		    ok3 = (y < INT64_MAX) ;
 		    I [k] = i - 1 ;
 		    n2 = MAX (n2, i) ;
 		}
@@ -1081,7 +1078,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 	for (i = 0 ; i < nrow ; i++) \
 	{ \
 	    xij = X [i + j*nrow] ; \
-	    if (CHOLMOD_IS_NONZERO (xij)) \
+	    if (xij != 0) \
 	    { \
 		nz++ ; \
 	    } \
@@ -1092,8 +1089,8 @@ Int sputil_copy_ij		/* returns the dimension, n */
 #define COPY_DENSE_TO_SPARSE(stype) \
 { \
     stype *Sx ; \
-    Sp = (Int *) mxGetJc (S) ; \
-    Si = (Int *) mxGetIr (S) ; \
+    Sp = (int64_t *) mxGetJc (S) ; \
+    Si = (int64_t *) mxGetIr (S) ; \
     Sx = (stype *) mxGetData (S) ; \
     nz = 0 ; \
     for (j = 0 ; j < ncol ; j++) \
@@ -1102,7 +1099,7 @@ Int sputil_copy_ij		/* returns the dimension, n */
 	for (i = 0 ; i < nrow ; i++) \
 	{ \
 	    xij = X [i + j*nrow] ; \
-	    if (CHOLMOD_IS_NONZERO (xij)) \
+	    if (xij != 0) \
 	    { \
 		Si [nz] = i ; \
 		Sx [nz] = (stype) xij ; \
@@ -1125,8 +1122,8 @@ Int sputil_copy_ij		/* returns the dimension, n */
 mxArray *sputil_dense_to_sparse (const mxArray *arg)
 {
     mxArray *S = NULL ;
-    Int *Sp, *Si ;
-    Int nrow, ncol, nz, i, j ;
+    int64_t *Sp, *Si ;
+    int64_t nrow, ncol, nz, i, j ;
 
     nrow = mxGetM (arg) ;
     ncol = mxGetN (arg) ;
@@ -1158,15 +1155,15 @@ mxArray *sputil_dense_to_sparse (const mxArray *arg)
 	    {
 		xij = X [i + j*nrow] ;
 		zij = Z [i + j*nrow] ;
-		if (CHOLMOD_IS_NONZERO (xij) || CHOLMOD_IS_NONZERO (zij))
+		if ((xij != 0) || (zij != 0))
 		{
 		    nz++ ;
 		}
 	    }
 	} 
 	S = mxCreateSparse (nrow, ncol, nz, mxCOMPLEX) ;
-	Sp = (Int *) mxGetJc (S) ;
-	Si = (Int *) mxGetIr (S) ;
+	Sp = (int64_t *) mxGetJc (S) ;
+	Si = (int64_t *) mxGetIr (S) ;
 	Sx = mxGetPr (S) ;
 	Sz = mxGetPi (S) ;
 	nz = 0 ;
@@ -1177,7 +1174,7 @@ mxArray *sputil_dense_to_sparse (const mxArray *arg)
 	    {
 		xij = X [i + j*nrow] ;
 		zij = Z [i + j*nrow] ;
-		if (CHOLMOD_IS_NONZERO (xij) || CHOLMOD_IS_NONZERO (zij))
+		if ((xij != 0) || (zij != 0))
 		{
 		    Si [nz] = i ;
 		    Sx [nz] = xij ;
@@ -1250,12 +1247,12 @@ mxArray *sputil_dense_to_sparse (const mxArray *arg)
 
 	    case mxINT64_CLASS:
 
-		DENSE_TO_SPARSE (INT64_T) ;
+		DENSE_TO_SPARSE (int64_t) ;
 		break ;
 
 	    case mxUINT64_CLASS:
 
-		DENSE_TO_SPARSE (unsigned INT64_T) ;
+		DENSE_TO_SPARSE (uint64_t) ;
 		break ;
 
 	    case mxSINGLE_CLASS:
@@ -1284,16 +1281,16 @@ mxArray *sputil_dense_to_sparse (const mxArray *arg)
 /* ========================================================================== */
 
 /* Convert a triplet form into a sparse matrix.  If complex, s must be double.
- * If real, s can be of any class.  Optionally creates a
+ * If real, s can be of any class.
  */
 
 cholmod_sparse *sputil_triplet_to_sparse
 (
-    Int nrow, Int ncol, Int nz, Int nzmax,
-    Int i_is_scalar, Int i, void *i_vector, mxClassID i_class,
-    Int j_is_scalar, Int j, void *j_vector, mxClassID j_class,
-    Int s_is_scalar, double x, double z, void *x_vector, double *z_vector,
-    mxClassID s_class, Int s_complex,
+    int64_t nrow, int64_t ncol, int64_t nz, int64_t nzmax,
+    int64_t i_is_scalar, int64_t i, void *i_vector, mxClassID i_class,
+    int64_t j_is_scalar, int64_t j, void *j_vector, mxClassID j_class,
+    int64_t s_is_scalar, double x, double z, void *x_vector, double *z_vector,
+    mxClassID s_class, int64_t s_complex,
     cholmod_common *cm
 )
 {
@@ -1301,8 +1298,8 @@ cholmod_sparse *sputil_triplet_to_sparse
     cholmod_triplet *T ;
     cholmod_sparse *S ;
     double *Tx, *Tz ;
-    Int *Ti, *Tj ;
-    Int k, x_patch ;
+    int64_t *Ti, *Tj ;
+    int64_t k, x_patch ;
 
     /* ---------------------------------------------------------------------- */
     /* allocate the triplet form */
@@ -1437,7 +1434,7 @@ cholmod_sparse *sputil_triplet_to_sparse
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    Tx [k] = (double) (((INT64_T *) x_vector) [k]) ;
+		    Tx [k] = (double) (((int64_t *) x_vector) [k]) ;
 		}
 		break ;
 
@@ -1445,7 +1442,7 @@ cholmod_sparse *sputil_triplet_to_sparse
 
 		for (k = 0 ; k < nz ; k++)
 		{
-		    Tx [k] = (double) (((unsigned INT64_T *) x_vector) [k]) ;
+		    Tx [k] = (double) (((uint64_t *) x_vector) [k]) ;
 		}
 		break ;
 
@@ -1522,8 +1519,8 @@ mxArray *sputil_copy_sparse (const mxArray *A)
     double aij, zij ;
     mxArray *S ;
     double *Ax, *Az, *Sx, *Sz ;
-    Int *Ap, *Ai, *Sp, *Si ;
-    Int anz, snz, p, j, nrow, ncol, pend ;
+    int64_t *Ap, *Ai, *Sp, *Si ;
+    int64_t anz, snz, p, j, nrow, ncol, pend ;
 
 #ifndef MATLAB6p1_OR_EARLIER
 
@@ -1540,8 +1537,8 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 
     nrow = mxGetM (A) ;
     ncol = mxGetN (A) ;
-    Ap = (Int *) mxGetJc (A) ;
-    Ai = (Int *) mxGetIr (A) ;
+    Ap = (int64_t *) mxGetJc (A) ;
+    Ai = (int64_t *) mxGetIr (A) ;
     anz = Ap [ncol] ;
 
     snz = 0 ;
@@ -1569,8 +1566,8 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 
 	/* allocate S */
 	S = mxCreateSparseLogicalMatrix (nrow, ncol, snz) ;
-	Sp = (Int *) mxGetJc (S) ;
-	Si = (Int *) mxGetIr (S) ;
+	Sp = (int64_t *) mxGetJc (S) ;
+	Si = (int64_t *) mxGetIr (S) ;
 	Sl = mxGetLogicals (S) ;
 
 	/* copy A into S, dropping zero entries */
@@ -1609,7 +1606,7 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 	{
 	    aij = Ax [p] ;
 	    zij = Az [p] ;
-	    if (CHOLMOD_IS_NONZERO (aij) || CHOLMOD_IS_NONZERO (zij))
+	    if ((aij != 0) || (zij != 0))
 	    {
 		snz++ ;
 	    }
@@ -1617,8 +1614,8 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 
 	/* allocate S */
 	S = mxCreateSparse (nrow, ncol, snz, mxCOMPLEX) ;
-	Sp = (Int *) mxGetJc (S) ;
-	Si = (Int *) mxGetIr (S) ;
+	Sp = (int64_t *) mxGetJc (S) ;
+	Si = (int64_t *) mxGetIr (S) ;
 	Sx = mxGetPr (S) ;
 	Sz = mxGetPi (S) ;
 
@@ -1632,7 +1629,7 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 	    {
 		aij = Ax [p] ;
 		zij = Az [p] ;
-		if (CHOLMOD_IS_NONZERO (aij) || CHOLMOD_IS_NONZERO (zij))
+		if ((aij != 0) || (zij != 0))
 		{
 		    Si [snz] = Ai [p] ;
 		    Sx [snz] = aij ;
@@ -1655,7 +1652,7 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 	for (p = 0 ; p < anz ; p++)
 	{
 	    aij = Ax [p] ;
-	    if (CHOLMOD_IS_NONZERO (aij))
+	    if (aij != 0)
 	    {
 		snz++ ;
 	    }
@@ -1663,8 +1660,8 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 
 	/* allocate S */
 	S = mxCreateSparse (nrow, ncol, snz, mxREAL) ;
-	Sp = (Int *) mxGetJc (S) ;
-	Si = (Int *) mxGetIr (S) ;
+	Sp = (int64_t *) mxGetJc (S) ;
+	Si = (int64_t *) mxGetIr (S) ;
 	Sx = mxGetPr (S) ;
 
 	/* copy A into S, dropping zero entries */
@@ -1676,7 +1673,7 @@ mxArray *sputil_copy_sparse (const mxArray *A)
 	    for (p = Ap [j] ; p < pend ; p++)
 	    {
 		aij = Ax [p] ;
-		if (CHOLMOD_IS_NONZERO (aij))
+		if (aij != 0)
 		{
 		    Si [snz] = Ai [p] ;
 		    Sx [snz] = aij ;
@@ -1701,8 +1698,8 @@ mxArray *sputil_sparse_to_dense (const mxArray *S)
 {
     mxArray *X ;
     double *Sx, *Sz, *Xx, *Xz ;
-    Int *Sp, *Si ;
-    Int nrow, ncol, i, j, p, pend, j2 ;
+    int64_t *Sp, *Si ;
+    int64_t nrow, ncol, i, j, p, pend, j2 ;
 
 #ifndef MATLAB6p1_OR_EARLIER
 
@@ -1718,8 +1715,8 @@ mxArray *sputil_sparse_to_dense (const mxArray *S)
 
     nrow = mxGetM (S) ;
     ncol = mxGetN (S) ;
-    Sp = (Int *) mxGetJc (S) ;
-    Si = (Int *) mxGetIr (S) ;
+    Sp = (int64_t *) mxGetJc (S) ;
+    Si = (int64_t *) mxGetIr (S) ;
 
 #ifndef MATLAB6p1_OR_EARLIER
 
@@ -1804,11 +1801,11 @@ void sputil_check_ijvector (const mxArray *arg)
 	/* the i and j arguments for sparse(i,j,s,...) can be sparse, but if so
 	 * they must have no zero entries. */
 	double mn, m, nz ;
-	Int *p, n ;
+	int64_t *p, n ;
 	m = (double) mxGetM (arg) ;
 	n =  mxGetN (arg) ;
 	mn = m*n ;
-	p = (Int *) mxGetJc (arg) ;
+	p = (int64_t *) mxGetJc (arg) ;
 	nz = p [n] ;
 	if (mn != nz)
 	{
@@ -1839,8 +1836,8 @@ void sputil_sparse
     mxArray *s_array ;
     cholmod_sparse *S, *Z ;
     cholmod_common Common, *cm ;
-    Int nrow, ncol, k, nz, i_is_scalar, j_is_scalar, s_is_sparse,
-	s_is_scalar, ilen, jlen, slen, nzmax, i, j, s_complex ;
+    int64_t nrow, ncol, k, nz, i_is_scalar, j_is_scalar, s_is_sparse,
+	s_is_scalar, ilen, jlen, slen, nzmax, i, j, s_complex, ndropped ;
     mxClassID i_class, j_class, s_class ;
 
     /* ---------------------------------------------------------------------- */
@@ -1874,6 +1871,9 @@ void sputil_sparse
 	/* S = sparse (A) where A is sparse or full */
 	/* ------------------------------------------------------------------ */
 
+	nrow = mxGetM (pargin [0]) ;
+	ncol = mxGetN (pargin [0]) ;
+
 	if (mxIsSparse (pargin [0]))
 	{
 
@@ -1904,11 +1904,11 @@ void sputil_sparse
 	/* S = sparse (m,n) */
 	/* ------------------------------------------------------------------ */
 
-	Int *Sp ;
+	int64_t *Sp ;
 	nrow = sputil_get_integer (pargin [0], FALSE, 0) ;
 	ncol = sputil_get_integer (pargin [1], FALSE, 0) ;
 	pargout [0] = mxCreateSparse (nrow, ncol, 1, mxREAL) ;
-	Sp = (Int *) mxGetJc (pargout [0]) ;
+	Sp = (int64_t *) mxGetJc (pargout [0]) ;
 	Sp [0] = 0 ;
 
     }
@@ -2028,7 +2028,7 @@ void sputil_sparse
 		s_class, s_complex,
 		cm) ;
 
-	/* set nzmax(A) to nnz(S), unless nzmax is specified on input */
+	/* set nzmax(S) to nnz(S), unless nzmax is specified on input */
 	if (nargin <= 5 && S != NULL)
 	{
 	    cholmod_l_reallocate_sparse (cholmod_l_nnz (S, cm), S, cm) ;
@@ -2037,12 +2037,18 @@ void sputil_sparse
 	if (nargout > 1)
 	{
 	    /* return a binary pattern of the explicit zero entries, for the
-	     * S = sparse(i,j,x, ...) form. */
+	     * [S Z] = sparse(i,j,x, ...) form. */
 	    Z = sputil_extract_zeros (S, cm) ;
 	}
 
 	/* drop explicit zeros from S */
-	sputil_drop_zeros (S) ;
+	ndropped = sputil_drop_zeros (S) ;
+
+	/* if entries dropped, set nzmax(S) to nnz(S), unless nzmax specified */
+	if (ndropped > 0 && nargin <= 5 && S != NULL)
+	{
+	    cholmod_l_reallocate_sparse (cholmod_l_nnz (S, cm), S, cm) ;
+	}
 
 	if (s_is_sparse)
 	{
@@ -2067,11 +2073,11 @@ void sputil_sparse
 	    mxLogical *s_logical ;
 	    pargout [0] = mxCreateSparseLogicalMatrix (0, 0, 0) ;
 	    s_logical = cholmod_l_malloc (S->nzmax, sizeof (mxLogical), cm) ;
-	    for (k = 0 ; k < (Int) (S->nzmax) ; k++)
+	    for (k = 0 ; k < (int64_t) (S->nzmax) ; k++)
 	    {
 		s_logical [k] = 1 ;
 	    }
-	    mxFree (mxGetData (pargout [0])) ;
+	    MXFREE (mxGetData (pargout [0])) ;
 	    mxSetData (pargout [0], s_logical) ;
 	    mexMakeMemoryPersistent (s_logical) ;
 	    k++ ;
@@ -2079,13 +2085,15 @@ void sputil_sparse
 	else
 
 #endif	
+        if (S->x == NULL) mexErrMsgTxt ("invalid sparse matrix") ;
 
 	if (mxIsComplex (pargin [2]))
 	{
 	    /* copy S into a MATLAB sparse complex double matrix */
+            if (S->z == NULL) mexErrMsgTxt ("invalid complex sparse matrix") ;
 	    pargout [0] = mxCreateSparse (0, 0, 0, mxCOMPLEX) ;
-	    mxFree (mxGetPr (pargout [0])) ;
-	    mxFree (mxGetPi (pargout [0])) ;
+	    MXFREE (mxGetPr (pargout [0])) ;
+	    MXFREE (mxGetPi (pargout [0])) ;
 	    mxSetPr (pargout [0], S->x) ;
 	    mxSetPi (pargout [0], S->z) ;
 	    mexMakeMemoryPersistent (S->x) ;
@@ -2107,8 +2115,8 @@ void sputil_sparse
 	mxSetM (pargout [0], S->nrow) ;
 	mxSetN (pargout [0], S->ncol) ;
 	mxSetNzmax (pargout [0], S->nzmax) ;
-	mxFree (mxGetJc (pargout [0])) ;
-	mxFree (mxGetIr (pargout [0])) ;
+	MXFREE (mxGetJc (pargout [0])) ;
+	MXFREE (mxGetIr (pargout [0])) ;
 	mxSetJc (pargout [0], S->p) ;
 	mxSetIr (pargout [0], S->i) ;
 	mexMakeMemoryPersistent (S->p) ;
